use core::fmt::Display;

pub type Result<T> = core::result::Result<T, Error>;

#[allow(dead_code)]
#[derive(Debug, thiserror::Error)]
pub enum Error {
    #[error("SSE/SSE2 not available on this CPU")]
    NoSSE,
    #[error("CpuId failed to retrieve features")]
    NoCpuIdFeatures,
    #[error("could not find aligned subslice in slice")]
    NoAlignedSubslice,
    #[error("No Mode found for this GOP")]
    NoModeFound,
    #[error("{0}")]
    FontError(&'static str),
    #[error(transparent)]
    UefiError(#[from] UefiError),
    #[error(transparent)]
    LayoutError(#[from] alloc::alloc::LayoutError),
    #[error(transparent)]
    AllocError(#[from] alloc::alloc::AllocError),
    #[error("FromStrWithBufError: {0:?}")]
    FromStrWithBufError(uefi::data_types::FromStrWithBufError),
}

#[derive(Debug, thiserror::Error)]
pub struct UefiError {
    inner: uefi::Error,
}

impl From<uefi::data_types::FromStrWithBufError> for Error {
    fn from(value: uefi::data_types::FromStrWithBufError) -> Self {
        Self::FromStrWithBufError(value)
    }
}

impl From<uefi::Error> for Error {
    fn from(inner: uefi::Error) -> Self {
        Self::UefiError(UefiError { inner })
    }
}

impl Display for UefiError {
    fn fmt(&self, f: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
        write!(f, "UefiStatus: {:?}", self.inner.status())
    }
}
